package org.holer.server.handler;

import java.util.List;

import org.holer.server.constant.HolerCode;
import org.holer.server.model.HolerException;
import org.holer.server.model.HolerResult;
import org.springframework.validation.FieldError;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;


@RestControllerAdvice
public class ExceptHandler {
    @ExceptionHandler({Exception.class})
    public HolerResult<?> handle(Exception e) {
        e.printStackTrace();
        return new HolerResult(HolerCode.ERR_INNER.value(), e.getMessage());
    }


    @ExceptionHandler({MethodArgumentNotValidException.class})
    public HolerResult<?> handle(MethodArgumentNotValidException e) {
        List<FieldError> fieldErrors = e.getBindingResult().getFieldErrors();
        StringBuilder errMsg = new StringBuilder();
        for (FieldError error : fieldErrors) {
            errMsg.append(error.getDefaultMessage() + "\n");
        }
        return new HolerResult(HolerCode.ERR_INNER.value(), errMsg.toString());
    }

    @ExceptionHandler({HolerException.class})
    public HolerResult<?> handle(HolerException e) {
        return new HolerResult(e.getCode());
    }
}
